"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tieLogger = exports.createMiddleware = exports.customRequestProps = exports.createLogger = exports.defaultLoggerOptions = exports.logLevel = exports.redactionsList = void 0;
const colorette_1 = require("colorette");
const lodash_1 = __importDefault(require("lodash"));
const debug_1 = __importDefault(require("debug"));
const pino_1 = __importDefault(require("pino"));
const pino_pretty_1 = __importDefault(require("pino-pretty"));
const express_pino_logger_1 = __importDefault(require("express-pino-logger"));
const uuid_1 = require("uuid");
const keys_1 = require("./keys");
const context_1 = require("./context");
const debug = (0, debug_1.default)('hyperstack:logger');
const devPretty = (0, pino_pretty_1.default)({
    sync: true,
    colorize: true,
    levelFirst: true,
    translateTime: true,
    singleLine: true,
    ignore: 'pid,hostname',
});
exports.redactionsList = [
    'access_token',
    'data.access_token',
    'data.*.access_token',
    'data.*.accessToken',
    'accessToken',
    'data.accessToken',
    'DATABASE_URL',
    'data.*.email',
    'data.email',
    'email',
    'event.headers.authorization',
    'data.hashedPassword',
    'data.*.hashedPassword',
    'hashedPassword',
    'host',
    'jwt',
    'data.jwt',
    'data.*.jwt',
    'JWT',
    'data.JWT',
    'data.*.JWT',
    'password',
    'data.password',
    'data.*.password',
    'params',
    'data.salt',
    'data.*.salt',
    'salt',
    'secret',
    'data.secret',
    'data.*.secret',
];
const logLevel = () => {
    if (typeof process.env.LOG_LEVEL !== 'undefined') {
        return process.env.LOG_LEVEL;
    }
    else if ((0, context_1.isProduction)()) {
        return 'warn';
    }
    else if ((0, context_1.isTest)()) {
        return 'silent';
    }
    else {
        return 'trace';
    }
};
exports.logLevel = logLevel;
exports.defaultLoggerOptions = {
    level: (0, exports.logLevel)(),
    redact: {
        paths: exports.redactionsList,
    },
};
const createLogger = (loggerOpts = {}) => {
    const options = lodash_1.default.merge({}, exports.defaultLoggerOptions, loggerOpts);
    debug('Logger Configuration');
    debug(`isProduction: ${(0, context_1.isProduction)()}`);
    debug(`isDevelopment: ${(0, context_1.isDevelopment)()}`);
    debug(`isTest: ${(0, context_1.isTest)()}`);
    debug(`logLevel: ${options.level}`);
    debug(`stream: ${!!options.stream}`);
    const pinoOpts = lodash_1.default.omit(options, ['stream', 'middleware']);
    const colors = (0, colorette_1.createColors)({ useColor: !(0, context_1.isProduction)() });
    let logger = null;
    if ((0, context_1.isProduction)()) {
        logger = (0, pino_1.default)(pinoOpts, options.stream);
    }
    else {
        debug('Logging to dev environment: pretty mode');
        logger = (0, pino_1.default)(pinoOpts, devPretty);
    }
    logger.colors = colors;
    return logger;
};
exports.createLogger = createLogger;
const customRequestProps = (req, _res) => {
    if (req.user) {
        const user = req.user;
        // sniff out a reasonable identifier
        const out = {};
        ['id', 'pid', 'email', 'username'].forEach((k) => {
            if (user[k]) {
                out[`user-${k}`] = user[k];
            }
        });
        return out;
    }
    return {};
};
exports.customRequestProps = customRequestProps;
const createMiddleware = (logger, middlewareConfig) => {
    return (0, express_pino_logger_1.default)(Object.assign({ logger, genReqId: (req) => req.id || (0, uuid_1.v4)().toString(), customProps: exports.customRequestProps }, (middlewareConfig || {})));
};
exports.createMiddleware = createMiddleware;
const tieLogger = (context) => {
    var _a;
    const loggerConfig = context.config().logger;
    const logger = (0, exports.createLogger)(loggerConfig);
    const middlewareConfig = loggerConfig === null || loggerConfig === void 0 ? void 0 : loggerConfig.middleware;
    const middleware = (0, exports.createMiddleware)(logger, middlewareConfig);
    const initializers = context.store().initializers.initializers;
    const { logger: theirLogger, middleware: theirMiddleware } = ((_a = initializers.beforeLogger) === null || _a === void 0 ? void 0 : _a.reduce((res, init) => {
        const out = init(res);
        return out || res;
    }, { logger, middleware })) || {};
    // user can replace impl. of colors, but if none
    // given, we want ours.
    if (theirLogger && !theirLogger.colors) {
        theirLogger.colors = (0, colorette_1.createColors)({ useColor: !(0, context_1.isProduction)() });
    }
    const selectedLogger = theirLogger || logger;
    const selectedMiddleware = theirMiddleware || middleware;
    context.update(keys_1.setters.logger, {
        logging: {
            middleware: selectedMiddleware,
        },
        logger: selectedLogger,
    });
    return logger;
};
exports.tieLogger = tieLogger;
